﻿// Upgrade.js
//  This module manages the database upgrades, adding tables and columns
//  
//  v0.13.x -> v0.14.x [August 3, 2016]
//      - Ensure: AnalogInputs has FormulaId, FormulaEnabled
//
//  Added in v0.20.18 [April 24, 2018]
//      - Ensure: SnmpHosts has SnmpVersion, AuthUser, AuthPassword, AuthProtocol, PrivProtocol, PrivPassword
//

// Dependencies and Globals
//#region Dependencies and Globals
var moment = require('moment');
//#endregion Dependencies and Globals

//#region Database Upgrade

exports.UpgradeDatabase = function (CrestInfo,db,logger) {

    logger.info("Upgrade.UpgradeDatabase: Upgrading database to Crest v" + CrestInfo.version);

    //  pre-v0.14.0 to current version
    //      Table AnalogInputs: add columns FormulaId, FormulaEnabled
    UpgradeAnalogInputsv14(db,logger);

    //  pre-v0.20.18 to current version (added April 24, 2018)
    //      Table SnmpHosts: add columns SnmpVersion, AuthUser, AuthPassword, AuthProtocol, PrivPassword, PrivProtocol
    UpgradeSnmpHostsv20(db,logger);
}

function UpgradeAnalogInputsv14(db,logger) {
    logger.info("Upgrade.UpgradeAnalogInputsv14: Upgrading AnalogInputs table for Formula support");
    db.all("PRAGMA table_info(AnalogInputs)", function (err, row) {
        if (err) {
            logger.error("Upgrade.UpgradeAnalogInputsv14: Error getting AnalogInputs table_info err=" + JSON.stringify(err));
        } else if (row) {
            //#region Check if FormulaId exists
            var FormulaIdColumnExists = false;
            for (var i = 0; i < row.length; i++) {
                if (row[i].name == "FormulaId") {
                    FormulaIdColumnExists = true;
                    break;
                }
            }
            //#endregion Check if FormulaId exists
            // The FormulaId column doesn't exist, add it and fill=-1
            //#region ALTER TABLE AnalogInputs ADD COLUMN FormulaId INTEGER
            if (!FormulaIdColumnExists) {
                db.exec("ALTER TABLE AnalogInputs ADD COLUMN FormulaId INTEGER", function (err) {
                    if (err !== null) {
                        logger.error('Upgrade.UpgradeAnalogInputsv14: ALTER TABLE AnalogInputs ADD COLUMN FormulaId err=' + JSON.stringify(err));
                    } else {
                        logger.info('Upgrade.UpgradeAnalogInputsv14: ALTER TABLE AnalogInputs ADD COLUMN FormulaId done');

                        //#region UPDATE AnalogInputs SET FormulaId=-1
                        db.exec('UPDATE AnalogInputs SET FormulaId=-1', function (err) {
                            if (err !== null) {
                                logger.error('Upgrade.UpgradeAnalogInputsv14: UPDATE AnalogInputs SET FormulaId=-1" err=' + JSON.stringify(err));
                            } else {
                                logger.info('Upgrade.UpgradeAnalogInputsv14: UPDATE AnalogInputs SET FormulaId=-1 done');

                                //#region Check if FormulaEnabled exists
                                var FormulaEnabledColumnExists = false;
                                for (var i = 0; i < row.length; i++) {
                                    if (row[i].name == "FormulaEnabled") {
                                        FormulaEnabledColumnExists = true;
                                        break;
                                    }
                                }
                                //#endregion Check if FormulaEnabled exists
                                // If FormulaEnabled doesn't exist, create column and fill=1
                                //#region ALTER TABLE AnalogInputs ADD COLUMN FormulaEnabled INTEGER
                                if (!FormulaEnabledColumnExists) {
                                    db.exec("ALTER TABLE AnalogInputs ADD COLUMN FormulaEnabled INTEGER", function (err) {
                                        if (err !== null) {
                                            logger.error('Upgrade.UpgradeAnalogInputsv14: ALTER TABLE AnalogInputs ADD COLUMN FormulaEnabled err=' + JSON.stringify(err));
                                        } else {
                                            logger.info('Upgrade.UpgradeAnalogInputsv14: ALTER TABLE AnalogInputs ADD COLUMN FormulaEnabled done');

                                            //#region UPDATE AnalogInputs SET FormulaEnabled=1
                                            db.exec('UPDATE AnalogInputs SET FormulaEnabled=1', function (err) {
                                                if (err !== null) {
                                                    logger.error('Upgrade.UpgradeAnalogInputsv14: UPDATE AnalogInputs SET FormulaEnabled=1" err=' + JSON.stringify(err));
                                                } else {
                                                    logger.info('Upgrade.UpgradeAnalogInputsv14: UPDATE AnalogInputs SET FormulaEnabled=1 done');
                                                }
                                            });
                                            //#endregion UPDATE AnalogInputs SET FormulaEnabled=1
                                        }
                                    });
                                }
                                //#endregion ALTER TABLE AnalogInputs ADD COLUMN FormulaEnabled INTEGER
                            }
                        });
                        //#endregion UPDATE AnalogInputs SET FormulaId=-1
                    }
                });
            }
            //#endregion ALTER TABLE AnalogInputs ADD COLUMN FormulaId INTEGER
        }
    });
}

function UpgradeSnmpHostsv20(db,logger) {
    logger.info("Upgrade.UpgradeSnmpHostsv20: Upgrading SnmpHosts table for SNMPv3 support");
    db.all("PRAGMA table_info(SnmpHosts)", function (err, row) {
        if (err) {
            logger.error("Upgrade.UpgradeSnmpHostsv20: Error getting SnmpHosts table_info err=" + JSON.stringify(err));
        } else if (row) {
            //#region Check if SnmpVersion exists
            var SnmpVersionColumnExists = false;
            for (var i = 0; i < row.length; i++) {
                if (row[i].name == "SnmpVersion") {
                    SnmpVersionColumnExists = true;
                    break;
                }
            }
            //#endregion Check if FormulaId exists
            // The SnmpVersion column doesn't exist, add it and fill=2
            //#region ALTER TABLE SnmpHosts ADD COLUMN SnmpVersion INTEGER
            if (!SnmpVersionColumnExists) {
                db.exec("ALTER TABLE SnmpHosts ADD COLUMN SnmpVersion INTEGER", function (err) {
                    if (err !== null) {
                        logger.error('Upgrade.UpgradeSnmpHostsv20: ALTER TABLE SnmpHosts ADD COLUMN SnmpVersion err=' + JSON.stringify(err));
                    } else {
                        logger.info('Upgrade.UpgradeSnmpHostsv20: ALTER TABLE SnmpHosts ADD COLUMN SnmpVersion done');

                        //#region UPDATE SnmpHosts SET SnmpVersion=2
                        db.exec('UPDATE SnmpHosts SET SnmpVersion=2', function (err) {
                            if (err !== null) {
                                logger.error('Upgrade.UpgradeSnmpHostsv20: UPDATE SnmpHosts SET SnmpVersion=2" err=' + JSON.stringify(err));
                            } else {
                                logger.info('Upgrade.UpgradeSnmpHostsv20: UPDATE SnmpHosts SET SnmpVersion=2 done');

                                // We can safely assume that if we needed to add the SnmpVersion column,
                                //  we also need to add columns for AuthUser, AuthPassword, AuthProtocol, PrivPassword, PrivProtocol

                                //#region ALTER TABLE SnmpHosts ADD COLUMN AuthUser TEXT
                                db.exec("ALTER TABLE SnmpHosts ADD COLUMN AuthUser TEXT", function (err) {
                                    if (err !== null) {
                                        logger.error('Upgrade.UpgradeSnmpHostsv20: ALTER TABLE SnmpHosts ADD COLUMN AuthUser err=' + JSON.stringify(err));
                                    } else {
                                        logger.info('Upgrade.UpgradeSnmpHostsv20: ALTER TABLE SnmpHosts ADD COLUMN AuthUser done');

                                        //#region UPDATE SnmpHosts SET AuthUser=""
                                        db.exec('UPDATE SnmpHosts SET AuthUser=""', function (err) {
                                            if (err !== null) {
                                                logger.error('Upgrade.UpgradeSnmpHostsv20: UPDATE SnmpHosts SET AuthUser="" err=' + JSON.stringify(err));
                                            } else {
                                                logger.info('Upgrade.UpgradeSnmpHostsv20: UPDATE SnmpHosts SET AuthUser="" done');
                                            }
                                        });
                                        //#endregion UPDATE SnmpHosts SET AuthUser=""
                                    }
                                });
                                //#endregion ALTER TABLE SnmpHosts ADD COLUMN AuthUser TEXT

                                //#region ALTER TABLE SnmpHosts ADD COLUMN AuthProtocol INT
                                db.exec("ALTER TABLE SnmpHosts ADD COLUMN AuthProtocol INT", function (err) {
                                    if (err !== null) {
                                        logger.error('Upgrade.UpgradeSnmpHostsv20: ALTER TABLE SnmpHosts ADD COLUMN AuthProtocol err=' + JSON.stringify(err));
                                    } else {
                                        logger.info('Upgrade.UpgradeSnmpHostsv20: ALTER TABLE SnmpHosts ADD COLUMN AuthProtocol done');

                                        //#region UPDATE SnmpHosts SET AuthProtocol=0
                                        db.exec('UPDATE SnmpHosts SET AuthProtocol=0', function (err) {
                                            if (err !== null) {
                                                logger.error('Upgrade.UpgradeSnmpHostsv20: UPDATE SnmpHosts SET AuthProtocol=0 err=' + JSON.stringify(err));
                                            } else {
                                                logger.info('Upgrade.UpgradeSnmpHostsv20: UPDATE SnmpHosts SET AuthProtocol=0 done');
                                            }
                                        });
                                        //#endregion UPDATE SnmpHosts SET AuthProtocol=0
                                    }
                                });
                                //#endregion ALTER TABLE SnmpHosts ADD COLUMN AuthProtocol INT

                                //#region ALTER TABLE SnmpHosts ADD COLUMN AuthPassword TEXT
                                db.exec("ALTER TABLE SnmpHosts ADD COLUMN AuthPassword TEXT", function (err) {
                                    if (err !== null) {
                                        logger.error('Upgrade.UpgradeSnmpHostsv20: ALTER TABLE SnmpHosts ADD COLUMN AuthPassword err=' + JSON.stringify(err));
                                    } else {
                                        logger.info('Upgrade.UpgradeSnmpHostsv20: ALTER TABLE SnmpHosts ADD COLUMN AuthPassword done');

                                        //#region UPDATE SnmpHosts SET AuthPassword=""
                                        db.exec('UPDATE SnmpHosts SET AuthPassword=""', function (err) {
                                            if (err !== null) {
                                                logger.error('Upgrade.UpgradeSnmpHostsv20: UPDATE SnmpHosts SET AuthPassword="" err=' + JSON.stringify(err));
                                            } else {
                                                logger.info('Upgrade.UpgradeSnmpHostsv20: UPDATE SnmpHosts SET AuthPassword="" done');
                                            }
                                        });
                                        //#endregion UPDATE SnmpHosts SET AuthPassword=""
                                    }
                                });
                                //#endregion ALTER TABLE SnmpHosts ADD COLUMN AuthPassword TEXT

                                //#region ALTER TABLE SnmpHosts ADD COLUMN PrivProtocol INT
                                db.exec("ALTER TABLE SnmpHosts ADD COLUMN PrivProtocol INT", function (err) {
                                    if (err !== null) {
                                        logger.error('Upgrade.UpgradeSnmpHostsv20: ALTER TABLE SnmpHosts ADD COLUMN PrivProtocol err=' + JSON.stringify(err));
                                    } else {
                                        logger.info('Upgrade.UpgradeSnmpHostsv20: ALTER TABLE SnmpHosts ADD COLUMN PrivProtocol done');

                                        //#region UPDATE SnmpHosts SET PrivProtocol=0
                                        db.exec('UPDATE SnmpHosts SET PrivProtocol=0', function (err) {
                                            if (err !== null) {
                                                logger.error('Upgrade.UpgradeSnmpHostsv20: UPDATE SnmpHosts SET PrivProtocol=0 err=' + JSON.stringify(err));
                                            } else {
                                                logger.info('Upgrade.UpgradeSnmpHostsv20: UPDATE SnmpHosts SET PrivProtocol=0 done');
                                            }
                                        });
                                        //#endregion UPDATE SnmpHosts SET PrivProtocol=0
                                    }
                                });
                                //#endregion ALTER TABLE SnmpHosts ADD COLUMN PrivProtocol INT

                                //#region ALTER TABLE SnmpHosts ADD COLUMN PrivPassword TEXT
                                db.exec("ALTER TABLE SnmpHosts ADD COLUMN PrivPassword TEXT", function (err) {
                                    if (err !== null) {
                                        logger.error('Upgrade.UpgradeSnmpHostsv20: ALTER TABLE SnmpHosts ADD COLUMN PrivPassword err=' + JSON.stringify(err));
                                    } else {
                                        logger.info('Upgrade.UpgradeSnmpHostsv20: ALTER TABLE SnmpHosts ADD COLUMN PrivPassword done');

                                        //#region UPDATE SnmpHosts SET PrivPassword=""
                                        db.exec('UPDATE SnmpHosts SET PrivPassword=""', function (err) {
                                            if (err !== null) {
                                                logger.error('Upgrade.UpgradeSnmpHostsv20: UPDATE SnmpHosts SET PrivPassword="" err=' + JSON.stringify(err));
                                            } else {
                                                logger.info('Upgrade.UpgradeSnmpHostsv20: UPDATE SnmpHosts SET PrivPassword="" done');
                                            }
                                        });
                                        //#endregion UPDATE SnmpHosts SET PrivPassword=""
                                    }
                                });
                                //#endregion ALTER TABLE SnmpHosts ADD COLUMN PrivPassword TEXT

                            }
                        });
                        //#endregion UPDATE SnmpHosts SET SnmpVersion=2
                    }
                });
            }
            //#endregion ALTER TABLE SnmpHosts ADD COLUMN SnmpVersion INTEGER
        }
    });
}
//#endregion Database Upgrade